import json
from Products.CMFPlone.PloneBatch import Batch
from Products.Five import BrowserView
from ZTUtils import make_query
from collective.csvsearcher.interfaces import ISearchForm
from collective.csvsearcher.interfaces import ISearchWidget
from collective.csvsearcher.interfaces import IStorage
from plone.batching.browser import PloneBatchView
from plone.z3cform import layout
from z3c.form import form, button
from z3c.form.browser.checkbox import CheckBoxFieldWidget
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.interface import alsoProvides
from zope.schema.interfaces import IVocabularyFactory


def show_country(country):
    if not country:
        return False
    if country.lower() == 'nederland':
        return False
    return True


class SearchForm(form.Form):
    fields = form.field.Fields(ISearchForm)
    ignoreContext = True
    method = 'get'
    # Needed for the mechanize testbrowser if we use 'get' method
    # instead of 'post':
    enctype = 'application/x-www-form-urlencoded'
    id = 'csvsearcher'
    prefix = 'csvsearcher'

    def update(self):
        if self.request.form.get(self.prefix + '.buttons.clear'):
            self.ignoreRequest = True
        self.fields['soort_lid'].widgetFactory = CheckBoxFieldWidget
        super(SearchForm, self).update()
        for widget in self.widgets.values():
            alsoProvides(widget, ISearchWidget)
        self.widgets['district_apotheek'].noValueMessage = ''
        # These are handled in the sorting bar so we hide them in the form:
        self.widgets['sort_on'].mode = 'hidden'
        self.widgets['sort_order'].mode = 'hidden'

    @button.buttonAndHandler(u"Zoek", name='search')
    def handleSearch(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return
        storage = IStorage(self.context)
        # self.results = storage.search(naam=self.request.form.get('naam', ''))
        self.results = storage.search(**data)

    @button.buttonAndHandler(u"Wis", name='clear')
    def handleClear(self, action):
        # Clear the form.
        # Done in the update method, otherwise we are too late.
        pass


class Searcher(layout.FormWrapper):

    form = SearchForm
    results = []
    batchnavigation = ''
    sortingbar = ''

    def update(self):
        super(Searcher, self).update()
        if hasattr(self.form_instance, 'results'):
            b_start = self.request.get('b_start', 0)
            b_size = self.request.get('b_size', 20)
            self.results = Batch(self.form_instance.results, int(b_size),
                                 int(b_start), orphan=0)
            # We show the batch navigation twice, so we calculate it once here.
            nav = getMultiAdapter(
                (self.context, self.request), name='csvbatchnavigation')
            self.batchnavigation = nav(self.results, anchor='csvresults')
            # Do the same for the bar with result count and sorting options.
            sortingbar = getMultiAdapter(
                (self.context, self.request), name='csvsortingbar')
            self.sortingbar = sortingbar(self.results, anchor='csvresults')

    def row_image(self, row):
        membership = row.get('soort_lid')
        src = alt = title = ''
        orig_membership = membership
        if not isinstance(membership, basestring):
            membership = ''
        membership = membership.lower()
        base = '++resource++collective.csvsearcher/'
        if membership.startswith('apotheker'):
            src = base + 'Apotheker_geen_lid.png'
            alt = title = 'Apotheker, geen lid'
        elif membership == 'gewoon lid':
            src = base + 'Gewoon_lid.png'
            alt = title = 'Gewoon lid'
        elif membership == 'kandidaat lid':
            src = base + 'Kandidaat_lid.png'
            alt = title = 'Kandidaat lid'
        elif membership.startswith('kandidaatlid'):
            src = base + 'Kandidaatlid_geslaagd.png'
            alt = title = 'Kandidaat lid, geslaagd'
        elif membership == 'buitengewoon lid':
            src = base + 'Buitengewoon_lid.png'
            alt = title = 'Buitengewoon lid'
        else:
            alt = title = orig_membership

        return {
            'src': src,
            'alt': alt,
            'title': title,
        }

    def show_apotheek_country(self, row):
        return show_country(row.get('land_apotheek'))

    def show_apotheker_country(self, row):
        return show_country(row.get('land_apotheker'))


class CSVBatchView(PloneBatchView):

    def __call__(self, batch, **kwargs):
        # html anchor
        self.anchor = kwargs.pop('anchor', '')
        return super(CSVBatchView, self).__call__(batch, **kwargs)

    @property
    def _current_sort_on_value(self):
        value = self.request.get('csvsearcher.widgets.sort_on', '')
        if not value:
            return ''
        if isinstance(value, list):
            # We get a list of one, which can look strange.
            value = value[0]
        return value

    @property
    def current_sort_on_title(self):
        current = self._current_sort_on_value
        if not current:
            return ''
        factory = getUtility(
            IVocabularyFactory, 'collective.csvsearcher.sort_on')
        return factory(self.context).getTerm(current).title

    def sort_options(self):
        current = self._current_sort_on_value
        factory = getUtility(
            IVocabularyFactory, 'collective.csvsearcher.sort_on')
        terms = []
        for term in factory(self.context):
            if term.value == current:
                continue
            terms.append(term)
        return terms

    def make_link(self, pagenumber=None, pagesize=None,
                  sort_on=None, sort_order=None):
        # Change from original:
        # - added pagesize parameter
        # - added sort_on and sort_order parameters
        # - added anchor
        form = self.request.form
        if self.batchformkeys:
            batchlinkparams = dict([(key, form[key])
                                    for key in self.batchformkeys
                                    if key in form])
        else:
            batchlinkparams = form.copy()
        if pagesize:
            batchlinkparams['b_size'] = pagesize
        if sort_on:
            batchlinkparams['csvsearcher.widgets.sort_on'] = sort_on
        if sort_order:
            batchlinkparams['csvsearcher.widgets.sort_order'] = sort_order

        # Note: we use the original pagesize from the batch here,
        # otherwise we would jump too much in the results.  The start
        # result shown with the current pagesize should be the start
        # result shown with the new pagesize.
        start = max(pagenumber - 1, 0) * self.batch.pagesize
        # But we do want to start at a multiple of the new batchsize.
        # For example, when we have start=20 with pagesize 30, the
        # link to previous results would not be shown.  So we round it
        # to zero.  start=40 would be rounded to 30.
        if pagesize:
            start = start - (start % pagesize)

        # Avoid unicode errors in the make_query call.
        for k, v in batchlinkparams.items():
            if isinstance(v, unicode):
                batchlinkparams[k] = v.encode('utf-8')

        link = '%s?%s' % (self.request.ACTUAL_URL, make_query(batchlinkparams,
                          {self.batch.b_start_str: start}))
        if self.anchor:
            link += '#' + self.anchor
        return link


class Values(BrowserView):
    # Returns values in json list.  Used as autocomplete source.

    def _get_matches(self, field, term=''):
        if term:
            term = term.lower()
        storage = IStorage(self.context)
        return json.dumps([x for x in getattr(storage, field)
                           if term in x.lower()])

    def sorteer_naam(self, term=''):
        return self._get_matches('sorteer_naam', term=term)

    def naam_apotheek(self, term=''):
        return self._get_matches('naam_apotheek', term=term)

    def plaats_apotheek(self, term=''):
        return self._get_matches('plaats_apotheek', term=term)

    def plaats_apotheker(self, term=''):
        return self._get_matches('plaats_apotheker', term=term)
