from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.statusmessages.interfaces import IStatusMessage
from collective.csvsearcher import _
from collective.csvsearcher.interfaces import IUploadForm
from collective.csvsearcher.interfaces import IStorage
from plone.z3cform import layout
from z3c.form import form, button


class ControlPanelFormWrapper(layout.FormWrapper):
    """Use this form as the plone.z3cform layout wrapper to get the control
    panel layout.
    """

    index = ViewPageTemplateFile('controlpanel_layout.pt')


class UploadForm(form.Form):
    """Upload a csv file.
    """
    fields = form.field.Fields(IUploadForm)
    ignoreContext = True

    @property
    def nrows(self):
        storage = IStorage(self.context)
        return storage.nrows

    @property
    def formatting(self):
        storage = IStorage(self.context)
        reader = storage.reader_class([])
        return reader.formatting()

    def applyChanges(self, data):
        if data['strict']:
            reader_kwargs = {
                'raise_exceptions': True,
                'ignore_extra_columns': False
            }
        else:
            reader_kwargs = {
                'raise_exceptions': False,
                'ignore_extra_columns': True
            }
        storage = IStorage(self.context)
        result = storage.store_csv(data['csvfile'], **reader_kwargs)
        if not result:
            IStatusMessage(self.request).addStatusMessage(
                _(u"No correct info found in CSV. "
                  u"Maybe wrong amount of columns?"), "error")
            return
        IStatusMessage(self.request).addStatusMessage(
            _(u"CSV stored."), "info")

    @button.buttonAndHandler(_(u"Upload"), name='upload')
    def handleUpload(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return
        self.applyChanges(data)

    @button.buttonAndHandler(_(u"Remove all data"), name='clear')
    def handleClear(self, action):
        storage = IStorage(self.context)
        storage.clear()
        IStatusMessage(self.request).addStatusMessage(
            _(u"All data has been removed."), "info")

Upload = layout.wrap_form(UploadForm, ControlPanelFormWrapper)
Upload.label = _(u"Upload csv file")
