from collective.csvsearcher import _
from zope import schema
from zope.interface import Attribute, Interface, directlyProvides
from zope.component import getUtility
from zope.schema.interfaces import IContextAwareDefaultFactory
from zope.schema.interfaces import IVocabularyFactory


def select_all_soort_lid(context):
    # Maybe simply a list like this:
    # return ['Kandidaat lid', 'Gewoon lid']
    factory = getUtility(
        IVocabularyFactory, 'collective.csvsearcher.soort_lid')
    vocabulary = factory(context)
    return [term.value for term in vocabulary]

directlyProvides(select_all_soort_lid, IContextAwareDefaultFactory)


class IUploadForm(Interface):

    csvfile = schema.Bytes(title=_(u'CSV file'))

    strict = schema.Bool(
        title=_(u'Strict'),
        description=_(u'Stop import when encountering warnings or errors.'),
        default=True)


class ISearchForm(Interface):

    # Somehow the message factory does not get picked up here, so we
    # do not use _('Field name').
    sorteer_naam = schema.TextLine(
        title=u'Naam', required=False)
    plaats_apotheker = schema.TextLine(
        title=u'Woonplaats', required=False)
    naam_apotheek = schema.TextLine(
        title=u'Apotheek', required=False)
    plaats_apotheek = schema.TextLine(
        title=u'Vestigingsplaats', required=False)
    district_apotheek = schema.Choice(
        title=u'District', required=False,
        vocabulary='collective.csvsearcher.districts')
    soort_lid = schema.List(
        title=u'Ledensoort', required=False,
        value_type=schema.Choice(
            vocabulary='collective.csvsearcher.soort_lid'),
        defaultFactory=select_all_soort_lid)
    # These may be hidden.
    sort_on = schema.Choice(
        title=u'Sorteer op', required=False,
        vocabulary='collective.csvsearcher.sort_on',
        default='sorteer_naam')
    sort_order = schema.Choice(
        title=u'Sorteervolgorde', required=False,
        values=['ascending', 'descending'],
        default='ascending')


class ISearchWidget(Interface):
    # marker interface for widgets in the search form
    pass


class IStorage(Interface):
    """Storage for the csv data.

    Expected is that we can store a list of dictionaries.
    """

    field_names = Attribute('List of field names in data.')
    field_names = Attribute('Number of stored rows.')
    reader_class = Attribute('Python class to read a csv.')

    def get_data():
        """Get all data."""

    def store(data):
        """Store data.

        Return True if successful, otherwise False.
        """

    def store_csv(data, **kwargs):
        """Store csv data.

        data should be a string and the method is responsible for
        calling 'splitlines' on it.

        The keyword arguments should be passed to the csv reader_class.

        Return True if successful, otherwise False.
        """

    def clear():
        """Remove the existing data."""

    def search(**kwargs):
        """Search the data using the keyword arguments.

        Suggested is to support sorting using the 'reverse' keyword
        argument and the sort_on keyword argument.
        """
